"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const buffer_1 = require("./buffer");
/**
 * Sends a network request and returns the recorded tape.
 */
function send(host, method, path, headers, body, timeout) {
    return __awaiter(this, void 0, void 0, function* () {
        const [scheme, hostnameWithPort] = host.split("://");
        const [hostname, port] = hostnameWithPort.split(":");
        const response = yield new Promise((resolve, reject) => {
            const requestOptions = {
                hostname,
                method,
                path,
                port,
                headers: Object.assign({}, headers, { host: hostname }),
                timeout
            };
            const proxyRequest = scheme === "http"
                ? http_1.default.request(requestOptions, resolve)
                : https_1.default.request(requestOptions, resolve);
            proxyRequest.on("error", reject);
            proxyRequest.write(body);
            proxyRequest.end();
        });
        const statusCode = response.statusCode || 200;
        const responseBody = yield new Promise(resolve => {
            const chunks = [];
            response.on("data", chunk => {
                chunks.push(buffer_1.ensureBuffer(chunk));
            });
            response.on("end", () => resolve(Buffer.concat(chunks)));
        });
        return {
            request: {
                method,
                path,
                headers,
                body
            },
            response: {
                status: {
                    code: statusCode
                },
                headers: response.headers,
                body: responseBody
            }
        };
    });
}
exports.send = send;
